@@ -34,11 +34,14 @@
 #endif
 #include <sys/types.h>
 #include <sys/stat.h>
+#ifdef HAVE_SYS_STATFS_H
+#include <sys/statfs.h>
+#endif
 
 #ifdef G_OS_WIN32
 #include <wchar.h>
 #include <direct.h>
-#include <glib/gwin32.h>
+#include <glib.h>
 #ifdef HAVE_IO_H
 #include <io.h>
 #endif
@@ -67,14 +70,35 @@ static int
 rename_wrapper (char const *oldfilename, char const *newfilename)
 {
 	int result = g_rename (oldfilename, newfilename);
-#ifdef G_OS_WIN32
-	if (result) {
-		/* Win32's rename does not unlink the target.  */
-		(void)g_unlink (newfilename);
-		result = g_rename (oldfilename, newfilename);
+	if (!result)
+		goto done;
+
+#ifdef HAVE_SYS_STATFS_H
+#ifdef HAVE_2ARG_STATFS
+	/* The FUSE file system does not unlink the target.  */
+	if (errno == EPERM) {
+		int save_errno = errno;
+		struct statfs buf;
+		if (statfs (newfilename, &buf) == 0 &&
+		    memcmp (&buf.f_type, "FUse", 4) == 0)
+			goto unlink_and_retry;
+		errno = save_errno;
 	}
 #endif
+#endif
+
+#ifdef G_OS_WIN32
+	/* Win32's rename does not unlink the target.  */
+	goto unlink_and_retry;
+#endif
+
+done:
 	return result;
+
+unlink_and_retry:
+	(void)g_unlink (newfilename);
+	result = g_rename (oldfilename, newfilename);
+	goto done;
 }
 
 static int
@@ -238,7 +262,7 @@ gsf_output_stdio_close (GsfOutput *output)
 			saved_errno = errno;
 		res = gsf_output_set_error (output,
 					    saved_errno,
-					    g_strerror (saved_errno));
+					    "%s", g_strerror (saved_errno));
 	} else {
 		/* Restore permissions.  There is not much error checking we
 		 * can do here, I'm afraid.  The final data is saved anyways.
@@ -319,7 +343,7 @@ gsf_output_stdio_seek (GsfOutput *output, gsf_off_t offset, GSeekType whence)
 	if (0 == fseek (stdio->file, loffset, stdio_whence))
 		return TRUE;
 #endif
-	return gsf_output_set_error (output, errno, g_strerror (errno));
+	return gsf_output_set_error (output, errno, "%s", g_strerror (errno));
 }
 
 static gboolean
@@ -339,7 +363,7 @@ gsf_output_stdio_write (GsfOutput *output,
 		written = fwrite (buffer + (num_bytes - remaining), 1, 
 				  remaining, stdio->file);
 		if ((written < remaining) && ferror (stdio->file) != 0)
-			return gsf_output_set_error (output, errno, g_strerror (errno));
+			return gsf_output_set_error (output, errno, "%s", g_strerror (errno));
 
 		remaining -= written;
 	}
